#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"


#define SCREEN_WIDTH    600
#define SCREEN_HEIGHT   600

CBlob::CBlob()
{
    pos.x = rand() % SCREEN_WIDTH;
    pos.y = rand() % SCREEN_HEIGHT;

    vel.x = ((rand() % 61) - 30) / 10.0f;
    vel.y = ((rand() % 61) - 30) / 10.0f;

    radius = 2000 + (rand() % 6000);
}

void CBlob::update()
{
    pos += vel;

    if (pos.x < 0)
    {
        pos.x = 0;
        vel.x = -vel.x;
    }

    if (pos.x > SCREEN_WIDTH - 1)
    {
        pos.x = SCREEN_WIDTH - 1;
        vel.x = -vel.x;
    }

    if (pos.y < 0)
    {
        pos.y = 0;
        vel.y = -vel.y;
    }

    if (pos.y > SCREEN_HEIGHT - 1)
    {
        pos.y = SCREEN_HEIGHT - 1;
        vel.y = -vel.y;
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Metaballs 6", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    srand(time(NULL));

    const int nbBlobs = 6;
    CBlob* blobs = new CBlob[nbBlobs];

    while (sys.isQuitRequested() == false)
    {
        for (int y = 0; y < SCREEN_HEIGHT; y++)
            for (int x = 0; x < SCREEN_HEIGHT; x++)
            {
                Vec2f p(x, y);

                float power = 0;

                for (int i = 0; i < nbBlobs; i++)
                {
                    Vec2f delta = p - blobs[i].pos;
                    power += blobs[i].radius / delta.mag();
                }

                if (power > 255)
                    power = 255;

                Color c;
                float angle = (power + 214) * M_PI / 128;
                c.r = 127 * (1 + sin(angle));
                c.g = 127 * (1 + sin(angle + 2 * M_PI / 3));
                c.b = 127 * (1 + sin(angle + 4 * M_PI / 3));

                gfx.setPixel(x, y, c);
            }

        for (int i = 0; i < nbBlobs; i++)
            blobs[i].update();

        gfx.render();
        sys.processEvents();
    }

    delete[] blobs;
    gfx.quit();

    return EXIT_SUCCESS;
}
